
/***************************************************************************
 *   Copyright (C) 2007 by Guy Rutenberg                                   *
 *   guyrutenberg@gmail.com                                                *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include <iostream>
#include <time.h>
#include "hugestring.h"
using namespace std;

timespec diff(timespec start, timespec end)
{
	timespec temp;
	if ((end.tv_nsec-start.tv_nsec)<0) {
		temp.tv_sec = end.tv_sec-start.tv_sec-1;
		temp.tv_nsec = 1000000000+end.tv_nsec-start.tv_nsec;
	} else {
		temp.tv_sec = end.tv_sec-start.tv_sec;
		temp.tv_nsec = end.tv_nsec-start.tv_nsec;
	}
	return temp;
}

int main()
{
	int size, i;
	int num[26];
	char *temp;
	string::iterator tmp;
	timespec start, end;

	for (i=0;i<26;i++)
		num[i]=0;

	size = hugestring.size();
	cout<<"String iteration benchmark"<<endl;
	cout<<"String length: "<<size<<endl;

	cout<<"iteration using indexes"<<endl;
	clock_gettime(CLOCK_PROCESS_CPUTIME_ID, &start);
	for(i=0; i<size; i++) {
		num[hugestring[i]-'a']++;
	}
	clock_gettime(CLOCK_PROCESS_CPUTIME_ID, &end);
	cout<<diff(start,end).tv_sec<<":"<<diff(start,end).tv_nsec<<endl<<endl;

	cout<<"iteration using indexes over C string"<<endl;
	
	clock_gettime(CLOCK_PROCESS_CPUTIME_ID, &start);
	i=0;
	temp =(char *) hugestring.c_str();
	while (temp[i]) {
		num[temp[i]-'a']++;
		i++;
	}
	clock_gettime(CLOCK_PROCESS_CPUTIME_ID, &end);
	cout<<diff(start,end).tv_sec<<":"<<diff(start,end).tv_nsec<<endl<<endl;

	cout<<"iteration using string::at()"<<endl;

	clock_gettime(CLOCK_PROCESS_CPUTIME_ID, &start);
	for (i=0; i<size; i++) {
		num[hugestring.at(i)-'a']++;
	}
	clock_gettime(CLOCK_PROCESS_CPUTIME_ID, &end);
	cout<<diff(start,end).tv_sec<<":"<<diff(start,end).tv_nsec<<endl<<endl;

	cout<<"iteration using pointers over C string"<<endl;
	
	clock_gettime(CLOCK_PROCESS_CPUTIME_ID, &start);
	temp =(char *) hugestring.c_str();
	while (*temp) {
		num[*temp-'a']++;
		temp++;
	}
	clock_gettime(CLOCK_PROCESS_CPUTIME_ID, &end);
	cout<<diff(start,end).tv_sec<<":"<<diff(start,end).tv_nsec<<endl<<endl;

	cout<<"iteration using iterators"<<endl;

	clock_gettime(CLOCK_PROCESS_CPUTIME_ID, &start);
	for(tmp=hugestring.begin(); tmp!=hugestring.end(); tmp++)
		num[*tmp-'a']++;
	clock_gettime(CLOCK_PROCESS_CPUTIME_ID, &end);
	cout<<diff(start,end).tv_sec<<":"<<diff(start,end).tv_nsec<<endl<<endl;
	
	return 0;
}
